from typing import Dict, Any

from pymail_io.pymail_io import AbstractPyMailIO, PyMailIO


class PyMailIOTask(AbstractPyMailIO, PyMailIO):

    def __init__(self, *args, **kwargs):
        super(PyMailIOTask, self).__init__(self, *args, **kwargs)

    def send_email(self, *, subject, body) -> Any:
        """
        :param subject:
        :param body:
        :return:
        """
        self.pytask.run()
        metadata = self.send_email_on_queue(subject, body)
        self.pytask.stop()
        return metadata

    def get_task(self, metadata):
        """
        :param metadata:
        :return:
        """
        return self.pytask.get_task(metadata)



from typing import Dict, Any
import asyncio

from pymail_io.pymail_io import AbstractPyMailIO, PyMailIO


class PymailIOAsync(AbstractPyMailIO, PyMailIO):

    async def send_email(self) -> Dict[str, Any]:
        """
        :return: Return the PytaskIO metadata dict
        """
        result = await asyncio.sleep(1)
        return result



import smtplib
import ssl
from typing import Dict, Any, Union, List
from datetime import datetime
from email.message import EmailMessage


def format_msg(
        subject: str,
        body: str,
        sender_email: str,
        receiver_email: Union[List[str], str],
) -> EmailMessage:
    """
    :param subject:
    :param body:
    :param sender_email:
    :param receiver_email:
    :return:
    """
    message = EmailMessage()
    message["Subject"] = subject
    message["From"] = sender_email
    if isinstance(receiver_email, list):
        message["To"] = ", ".join(receiver_email)
    else:
        message["To"] = receiver_email
    message.preamble = f"{body}"
    return message


def unit_of_work_callable(
        sender_email: str,
        password: str,
        receiver_email: str,
        host: str,
        port: int,
        subject: str,
        body: str,
) -> Dict[str, Any]:
    """
    :param sender_email:
    :param password:
    :param receiver_email:
    :param host:
    :param port:
    :param subject:
    :param body:
    :return:
    """
    time_sent = None
    _SSL_CONTEXT = ssl.create_default_context()
    message = format_msg(subject, body, sender_email, receiver_email)
    with smtplib.SMTP_SSL(host, port, context=ssl.SSLContext()) as server:
        try:
            server.login(sender_email, password)
            time_sent = datetime.now()
            server.send_message(message)
        except smtplib.SMTPAuthenticationError as err:
            Warning(
                "PyMailIO Error: Couldn't authenticate email senders credentials"
            )
        finally:
            server.quit()
    return {
        "sent_email": {
            "subject": subject,
            "body": body,
            "time_sent": time_sent,
        }
    }


from typing import Dict, Any
import asyncio

from pymail_io.pymail_io import AbstractPyMailIO, PyMailIO


class PyMailIOSync(AbstractPyMailIO, PyMailIO):

    def __init__(self, *args, **kwargs):
        super(PyMailIOSync, self).__init__(self, *args, **kwargs)
        self.init()

    def send_email(self, *, subject, body) -> Dict[str, None]:
        """
        The response will always be None
        :param subject: The email title or subject
        :param body: The text body of the email
        :return: The response is a Dict with a `response` key
        """
        res = self.create_and_send_email(subject, body)
        return res





"""
PyMailIO is a low level class designed to be used be authors of libraries etc.
"""
from abc import ABC, abstractmethod
import smtplib
import ssl

from typing import Dict, Any
from pytask_io import PyTaskIO
import asyncio
import time
import threading

from pymail_io._email import Email
from pymail_io._callables import unit_of_work_callable


class AbstractPyMailIO(ABC):

    @abstractmethod
    def send_email(self, *, subject, body) -> Dict[str, Any]:
        pass


class PyMailIO:
    """
    :kwargs:
    :key password: Your host email password.
    :key receiver_email: This can be either a string or a list of email addresses.
    :key sender_email: The sender email.
    :key store_port: Redis store port (defaults to 6379).
    :key store_host: The email server host.
    :key db: The Redis store database name.
    :key workers: The number of workers created to run tasks in the queue.
    :key host: The email server host.
    :key port: The email server SSL or TLS port.
    """

    #: PyMailIO is a python library built on CPython's AsyncIO library.
    #: The entree to asyncio is via `PyTaskIO https://github.com/joegasewicz/pytask-io` which is
    #: an asynchronous task queue library that runs an event loop in a background thread.
    #:
    #: Setting up the library for debugging. Example::
    #:
    #:  export PYTASKIO_DEBUG=1
    #:
    #:
    password: str

    receiver_email: str

    sender_email: str

    store_port: int

    store_host: str

    db: int

    workers: int

    host: str

    pytask: PyTaskIO = None

    queue: PyTaskIO

    email: Email

    _SMPT_SSL_PORT = 465
    _START_TLS_PORT = 587

    def __init__(self, *args, **kwargs):
        self.password = kwargs.get("password")
        self.receiver_email = kwargs.get("receiver_email")
        self.sender_email = kwargs.get("sender_email")
        self.store_port = kwargs.get("store_port")
        self.store_host = kwargs.get("store_host")
        self.db = kwargs.get("db")
        self.workers = kwargs.get("workers")
        self.host = kwargs.get("host")
        self.port = kwargs.get("port") or self._SMPT_SSL_PORT
        self.pytask = PyTaskIO(
            store_port=6379,
            store_host="localhost",
            db=0,
            workers=3,
        )
        self.email = Email(
            queue=self.pytask,
            sender_email=self.sender_email,
            password=self.password,
            receiver_email=self.receiver_email,
            host=self.host,
            port=self.port,
        )

    def send_email_sync(self, email_msg: str):
        """
        :param email_msg:
        :return:
        """
        def inner():
            server = smtplib.SMTP_SSL(self.host, self._SMPT_SSL_PORT)
            try:
                server.login(self.sender_email, self.password)
                server.sendmail(self.sender_email, self.receiver_email, email_msg)
            except smtplib.SMTPAuthenticationError as err:
                Warning(
                    "PyMailIO Error: Couldn't authenticate email senders credentials"
                )

            server.quit()
        return inner

    def create_and_send_email(self, subject: str, body: str) -> Any:
        """
        :param subject:
        :param body:
        :return:
        """
        msg = self.format_msg(subject, body)
        self.send_email_sync(msg)

    def send_email_on_queue(self, subject: str, body: str):
        """
        :param subject:
        :param body:
        :return:
        """
        self.email.add_email_to_task_queue(unit_of_work_callable, [subject, body])



"""
Email class
"""
from pytask_io import PyTaskIO
from typing import Dict, Any, Callable
from functools import partial


class Queue(PyTaskIO):
    """
        DI here
    """
    pass


class Email:

    queue: PyTaskIO
    sender_email: str
    password: str
    receiver_email: str
    host: str
    port: int

    def __init__(
            self,
            queue,
            sender_email: str,
            password: str,
            receiver_email: str,
            host: str,
            port: int,
    ):
        self.queue = queue
        self.sender_email = sender_email
        self.password = password
        self.receiver_email = receiver_email
        self.host = host
        self.port = port

    def add_email_to_task_queue(
            self,
            unit_of_work: Callable,
            email_data: Any,
    ) -> Dict[str, Any]:
        """
        :param email_data:
        :param unit_of_work:
        :return:
        """
        callable_uow = partial(
            unit_of_work,
            self.sender_email,
            self.password,
            self.receiver_email,
            self.host,
            self.port,
        )

        subject, body = email_data
        meta_data = self.queue.add_task(callable_uow, subject, body)
        return meta_data



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name="pymail-io",
    version="0.0.2",
    description="An asynchronous mail server",
    packages=['pymail_io'],
    install_requires=[
        "pytask-io",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/joegasewicz/pymail_io",
    author="Joe Gasewicz",
    author_email="joegasewicz@gmail.com",
)
